module App.ResourceRecord where

import Data.Maybe (Maybe)

import Data.Codec.Argonaut (JsonCodec)
import Data.Codec.Argonaut as CA
import Data.Codec.Argonaut.Record as CAR

type ResourceRecord
  = { rrtype   :: String
    , rrid     :: Int
    , name     :: String
    , ttl      :: Int
    , target   :: String
    , readonly :: Boolean

    -- MX (and SRV) specific entry.
    , priority :: Maybe Int

    -- SRV specific entries.
    , port     :: Maybe Int
    , protocol :: Maybe String
    , weight   :: Maybe Int

    -- SOA specific entries.
    , mname   :: Maybe String
    , rname   :: Maybe String
    , serial  :: Maybe Int
    , refresh :: Maybe Int
    , retry   :: Maybe Int
    , expire  :: Maybe Int
    , minttl  :: Maybe Int
    }

codec :: JsonCodec ResourceRecord
codec = CA.object "ResourceRecord"
  (CAR.record
    { rrtype:   CA.string
    , rrid:     CA.int
    , name:     CA.string
    , ttl:      CA.int
    , target:   CA.string
    , readonly: CA.boolean

    -- MX (and SRV) specific entry.
    , priority: CAR.optional CA.int

    -- SRV specific entries.
    , port:     CAR.optional CA.int
    , protocol: CAR.optional CA.string
    , weight:   CAR.optional CA.int

    -- SOA specific entries.
    , mname:   CAR.optional CA.string
    , rname:   CAR.optional CA.string
    , serial:  CAR.optional CA.int
    , refresh: CAR.optional CA.int
    , retry:   CAR.optional CA.int
    , expire:  CAR.optional CA.int
    , minttl:  CAR.optional CA.int
    })
